# Generated by Django 4.0.3 on 2022-04-07 03:40

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ipam', '0057_created_datetimefield'),
        ('netbox_config_backup', '0009_bctc_file_model_backup_fk_restructure'),
    ]

    operations = [
        migrations.AddField(
            model_name='backup',
            name='ip',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='ipam.ipaddress'),
        ),
    ]
