{% extends 'generic/object_children.html' %}
{% load helpers %}

{% block subtitle %}
  <div class="object-subtitle"></div>
{% endblock %}

{%  block extra_controls %}
  <div class="btn-group" role="group">
    <a href="{{ request.path }}{% querystring request type='startup' %}" class="btn btn-sm {% if startup and not running %}btn-secondary active{% else %}btn-outline-secondary{% endif %}">
      Show Startup
    </a>
    <a href="{{ request.path }}{% querystring request type='running' %}" class="btn btn-sm {% if running and not startup %}btn-secondary active{% else %}btn-outline-secondary{% endif %}">
      Show Running
    </a>
    <a href="{{ request.path }}{% querystring request type='' %}" class="btn btn-sm {% if not running and not startup %}btn-secondary active{% else %}btn-outline-secondary{% endif %}">
      Show All
    </a>
  </div>
{% endblock %}

{% block bulk_edit_controls %}
    {{ block.super }}
    {% with diff_view=object|viewname:"diff" %}

            <button type="submit" name="_diff"
                    formaction="{% url diff_view backup=object.pk %}?return_url={{ return_url }}"
                    class="btn btn-warning btn-sm">
                <i class="mdi mdi-file-compare" aria-hidden="true"></i> Diff Selected (Max 2)
            </button>
    {% endwith %}
{% endblock bulk_edit_controls %}
